@@ -1,5 +1,4 @@
 from . import exceptions
-from typing import Union
 
 
 class HTTPData(object):
@@ -72,7 +71,7 @@ class HTTPParser(object):
                 name, value = header.split(':', maxsplit=1)
                 value = value.strip()
 
-                if ';' in value and ',' in value:
+                if ',' in value or ';' in value and name != 'Cookie':
                     sub_values = value.split(',')
                     filtered_values = []
 
@@ -82,10 +81,7 @@ class HTTPParser(object):
 
                     parsed_headers[name] = filtered_values
                 else:
-                    if name == 'Cookie':
-                        self.result.cookies = self._parser_cookies(value)
-                    else:
-                        parsed_headers[name] = value
+                    parsed_headers[name] = value
 
         return parsed_headers
 
@@ -113,7 +109,11 @@ class HTTPParser(object):
         except (ValueError, IndexError):
             raise exceptions.InvalidHTTPMessageError('Invalid HTTP message')
 
-        self.result.body = msg_parts[-1]
+        if msg_parts[-1]:
+            self.result.body = msg_parts[-1]
+
+        msg_parts.pop()
+        msg_parts.pop()
 
         self.result.method = method
         self.result.path = path
@@ -121,12 +121,17 @@ class HTTPParser(object):
 
         self.result.real_path = self.result.path
         self.result.query = self._parser_query_string(path)
-        headers = self._parser_headers(msg_parts)
-        
-        self.result.host = headers.get('Host')
-        self.result.user_agent = headers.get('User-Agent')
-        self.result.accept = headers.get('Accept')
-        self.result.headers = headers
+        self.result.headers = self._parser_headers(msg_parts)
+
+        self.result.host = self.result.headers.get('Host')
+        self.result.user_agent = self.result.headers.get('User-Agent')
+        self.result.accept = self.result.headers.get('Accept')
+
+        cookies = self.result.headers.get('Cookie')
+
+        if cookies:
+            self.result.cookies = self._parser_cookies(cookies)
+            self.result.headers.pop('Cookie')
 
         return self.result
 
